/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Tom Wheeler. Portions Copyright 2006 Tom Wheeler. All Rights 
 * Reserved. Any future updates to this code will be made available at 
 * http://www.tomwheeler.com/netbeans/
 */

package com.tomwheeler.example.tdlfile;

import com.tomwheeler.example.tdlfile.gui.EditSingleItemAction;
import com.tomwheeler.example.tdlfile.intf.TodoListItem;
import java.lang.reflect.InvocationTargetException;
import java.util.Date;
import javax.swing.Action;
import org.openide.actions.DeleteAction;
import org.openide.actions.PropertiesAction;
import org.openide.actions.RenameAction;
import org.openide.nodes.*;
import org.openide.util.Lookup;
import org.openide.util.actions.SystemAction;

/**
 * <p>
 * This class is a NetBeans node that shows a single item within a Todo List.
 * </p>
 * 
 * @author <a href="http://www.tomwheeler.com/netbeans/">Tom Wheeler</a>
 */
public class SingleItemNode extends AbstractNode {
    
    private static final String IMAGE_ICON_BASE = "com/tomwheeler/example/tdlfile/resources/tdlitem.png";
    
    private TodoListItem item;
    
    public SingleItemNode(TodoListItem item, Lookup lookup) {
        super(Children.LEAF, lookup);
        this.item = item;
        setIconBaseWithExtension(IMAGE_ICON_BASE);
        setName(item.getName());
    }
    
    /** defines what actions are available on this node (i.e. in the context menu) */
    public Action[] getActions(boolean context) {
        return new Action[] {
            SystemAction.get(DeleteAction.class),
            SystemAction.get(RenameAction.class),
            SystemAction.get(PropertiesAction.class),
            SystemAction.get(EditSingleItemAction.class), // invokes my custom editor
        };
    }
    
    /** specifies what happens on double-click */
    public Action getPreferredAction() {
        return SystemAction.get(PropertiesAction.class);
    }

    /** 
     * build the property sheet that is displayed when the properties window is
     * open and a node of this type is selected.  This is also where we create
     * the properties for the node.  It works, but I am not sure if it's the best
     * place to create them,
     */
    protected Sheet createSheet() {
        Sheet s = super.createSheet();
        Sheet.Set ss = s.get(Sheet.PROPERTIES);
        if (ss == null) {
            ss = Sheet.createPropertiesSet();
            s.put(ss);
        }
        
        // add our properties        
        ss.put(new ItemDescriptionProperty());        
        ss.put(new ItemDueDateProperty());
        
        // node names are a special case
        ss.put(new PropertySupport.Name(this));
                                                                                                                                                 
        return s;
    }
    
    /** 
     * we allow the node to be renamed; by default this is false for AbstractNode.
     * This, by itself, has no effect, so see the property change listener in 
     * the createNodes() method of the Children.Keys inside the TodoListFileDataNode
     * to see how I listen for and react to changes in the node's name property,
     */
    public boolean canRename() {
        return true;
    }
    
    /** allows nodes of this type to be deleted. See commentary in canRename() */
    public boolean canDestroy() {
        return true;
    }

    /** the property representing our item's description */
    private class ItemDescriptionProperty extends PropertySupport.ReadWrite {
        
        public ItemDescriptionProperty() {
            super(TodoListItem.PROP_DESC, String.class, "Description", "A brief description of this item");
        }
        
        /** returns the current value of the TodoListItem's description */
        public Object getValue() throws IllegalAccessException, InvocationTargetException {
            return item.getDesc();
        }

        /** changes the current value of the TodoListItem's description */
        public void setValue(Object newValue) throws IllegalAccessException, IllegalArgumentException, InvocationTargetException {
            // note that we don't need to set up any listeners here, since any modification
            // to the item will fire an event to anything listening to changes on that item
            // (in other words, firePropertyChange() is called in the item's setDesc method)
            String oldDesc = item.getDesc();
            item.setDesc((String)newValue);
        }
    }

    /** the property representing our item's due date */
    private class ItemDueDateProperty extends PropertySupport.ReadWrite {

        public ItemDueDateProperty() {
            super(TodoListItem.PROP_DUE_DATE, Date.class, "Due Date", "Date by which this item must be completed");
        }
        
        /** returns the current value of the TodoListItem's due date */
        public Object getValue() throws IllegalAccessException, InvocationTargetException {
            return item.getDueDate();
        }
        
        /** changes the current value of the TodoListItem's due date */
        public void setValue(Object newValue) throws IllegalAccessException, IllegalArgumentException, InvocationTargetException {
            // commentary in ItemDescriptionProperty.setValue applies here, too
            Date oldDueDate = item.getDueDate();
            item.setDueDate((Date)newValue);
        }
    }
}
