/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Tom Wheeler. Portions Copyright 2006 Tom Wheeler. All Rights 
 * Reserved. Any future updates to this code will be made available at 
 * http://www.tomwheeler.com/netbeans/
 */

package com.tomwheeler.example.tdlfile.gui;

import com.tomwheeler.example.tdlfile.intf.TodoListItem;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import org.openide.ErrorManager;

/**
 * <p>
 * This class controls the editor for a single item on the TODO list.  The 
 * view that it controls is defined in the <code>TodoListItemEditorForm</code>
 * class.  This is just my preference for working with forms using the NetBeans
 * form editor (Matisse).  Other people may prefer to put logic directly into
 * the view class.  Either way will work, though.  Also note that there is 
 * nothing specific to NetBeans in this class; it is just pure Swing code.
 * The only important things is that we listen to events fired on changes to
 * the TodoListItem we're editing.
 * </p>
 *
 * @author <a href="http://www.tomwheeler.com/netbeans/">Tom Wheeler</a>
 */
public class TodoListItemEditorController implements PropertyChangeListener {
    
    private TodoListItem editedItem;
    private TodoListItemEditorForm form;
    
    /**
     * Creates a new instance of TodoListItemEditorController
     */
    public TodoListItemEditorController() {
        form = new TodoListItemEditorForm();
        
        form.getCancelButton().addActionListener(new ActionListener() {
            // reset form fields to item's current state if the cancel button is clicked
            public void actionPerformed(ActionEvent e) {
                updateFormWithCurrentState();
            }
        });
        
        form.getUpdateButton().addActionListener(new ActionListener() {
            // change the currednt item's state if the update button is clicked
            public void actionPerformed(ActionEvent e) {
                // we have to avoid getting caught in a loop, which could happen
                // if we react to the changes we're making.  We can work around
                // this by removing ourselves as a listener, making the changes
                // and then add ourselves again.  Note that you don't have to do
                // these stupid tricks if you're using the JGoodies DataBinding
                // library.
                removeSelfAsPropertyChangeListener();
                
                editedItem.setName(form.getNameField().getText());
                editedItem.setDesc(form.getDescField().getText());
                
                try {
                    // date might be bad, so we have to handle this possibility
                    String dueDateText = form.getDueDateField().getText();
                    Date dueDate = SimpleDateFormat.getDateTimeInstance().parse(dueDateText);
                    editedItem.setDueDate(dueDate);
                } catch (ParseException pe) {
                    ErrorManager.getDefault().notify(pe);
                }
                
                // now we're 
                addSelfAsPropertyChangeListener();
            }            
        });

    }
    
    private void updateFormWithCurrentState() {
        setName(editedItem.getName());
        setDesc(editedItem.getDesc());
        setDueDate(editedItem.getDueDate());
    }
    
    TodoListItemEditorForm getForm() {
        return form;
    }
    
    private void removeSelfAsPropertyChangeListener() {
        if (editedItem != null) {
            editedItem.removePropertyChangeListener(this);
        }
    }
    
    private void addSelfAsPropertyChangeListener() {
        if (editedItem != null) {
            editedItem.addPropertyChangeListener(this);
        }
    }
    
    public void setTodoListItem(TodoListItem editedItem) {
        // we no longer want to listen to changes on old item, if there was one
        removeSelfAsPropertyChangeListener();
        
        this.editedItem = editedItem;        

        updateFormWithCurrentState();
        
        // stay aware of any changes made to the TodoListItem externally (e.g. 
        // from the table view or property editor)
        addSelfAsPropertyChangeListener();
    }
    
    public TodoListItem getTodoListItem() {
        return editedItem;
    }
    
    public void propertyChange(PropertyChangeEvent e) {
        /* 
         * NOTE: I want to keep this example simple and not introduce any extra
         * dependencies.  However, for a real-life application, you should 
         * consider using the JGoodies DataBinding library to handle event
         * notification and GUI updates for bound properties.  See my 'Dogfile 
         * Example' module at http://www.tomwheeler.com/netbeans/ for an example 
         * using NetBeans, or see https://databinding.dev.java.net/ for more info
         * on the JGoodies DataBinding library.
         */
        
        if (TodoListItem.PROP_NAME.equals(e.getPropertyName())) {
            setName(editedItem.getName());
        } else if (TodoListItem.PROP_DESC.equals(e.getPropertyName())) {
            setDesc(editedItem.getDesc());
        } else if (TodoListItem.PROP_DUE_DATE.equals(e.getPropertyName())) {
            setDueDate(editedItem.getDueDate());
        } else {
            // should not happen unless you added new fields to the TodoListItem
            // class and forgot to account for them here
            System.err.println("unhandled TodoListItem property " + e.getPropertyName() + " changed.");
        }        
    }
    
    private void setName(String name) {
        form.getNameField().setText(name);
    }
    
    private void setDesc(String desc) {
        form.getDescField().setText(desc);
    }
    
    private void setDueDate(Date dueDate) {
        if (dueDate == null) {
            form.getDueDateField().setText(null);
        } else {
            // Probably not the best way to do this, but it's getting late...
            DateFormat format = SimpleDateFormat.getDateTimeInstance();
            String formattedDueDate = format.format(dueDate);
            form.getDueDateField().setText(formattedDueDate);            
        }
    }
}
