/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Tom Wheeler. Portions Copyright 2006 Tom Wheeler. All Rights 
 * Reserved. Any future updates to this code will be made available at 
 * http://www.tomwheeler.com/netbeans/
 */

package com.tomwheeler.example.tdlfile.gui;

import java.awt.BorderLayout;
import java.io.IOException;
import java.io.Serializable;
import javax.swing.ActionMap;
import javax.swing.text.DefaultEditorKit;
import org.openide.ErrorManager;
import org.openide.explorer.ExplorerManager;
import org.openide.explorer.ExplorerUtils;
import org.openide.explorer.view.BeanTreeView;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileSystem;
import org.openide.filesystems.Repository;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.nodes.Node;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;
import org.openide.windows.TopComponent;
import org.openide.windows.WindowManager;

/**
 * <p>
 * This class is a TopComponent that shows the available TodoLists.  The first
 * time that you start the application, or after you do a 'clean' build, you
 * will not have any Todo lists and will need to create one by right-clicking
 * (ctrl-click for Mac OS X) and choosing the appropriate option under the 
 * 'New' item in the context menu.
 * </p>
 *
 * @author <a href="http://www.tomwheeler.com/netbeans/">Tom Wheeler</a>
 */
final class TodoListManagerTopComponent extends TopComponent implements ExplorerManager.Provider {
    
    private static final long serialVersionUID = 1L;
    
    private transient Node exploredNode;
    private transient BeanTreeView beanTreeView;
    private transient ExplorerManager explorerManager;
    
    
    private static TodoListManagerTopComponent instance;
    
    /** path to the icon used by the component and its open action */
    static final String ICON_PATH = "com/tomwheeler/example/tdlfile/resources/tdlfile.png";
    
    private static final String PREFERRED_ID = "TodoListManagerTopComponent";
    
    private TodoListManagerTopComponent() {
        initComponents();
        setName(NbBundle.getMessage(TodoListManagerTopComponent.class, "CTL_TodoListManagerTopComponent"));
        setToolTipText(NbBundle.getMessage(TodoListManagerTopComponent.class, "HINT_TodoListManagerTopComponent"));
        setIcon(Utilities.loadImage(ICON_PATH, true));
        
        setLayout(new BorderLayout());
        beanTreeView = new BeanTreeView();
        add(beanTreeView, BorderLayout.CENTER);
        
        
        // this bit of code gets a reference to the folder within the system filesystem
        // where our TodoList files are stored.  If it does not exist, it will be created.
        String todoListRootName = "TodoListFiles";
        FileSystem systemFs = Repository.getDefault().getDefaultFileSystem();
        FileObject todoListRoot = systemFs.findResource("/" + todoListRootName); // a path
        if (todoListRoot == null) {
            try {
                todoListRoot = systemFs.getRoot().createFolder(todoListRootName);
            } catch (IOException ioe) {
                ErrorManager.getDefault().notify(ioe);
                return;
            }
        }
        
        DataObject todoListRootDataObj;
        try {
            todoListRootDataObj = DataObject.find(todoListRoot);
        } catch (DataObjectNotFoundException dnfe) {
            ErrorManager.getDefault().notify(dnfe);
            return;
        }
        
        exploredNode = todoListRootDataObj.getNodeDelegate();
    
        explorerManager = new ExplorerManager();
        
        
        beanTreeView.setDragSource(true);
        beanTreeView.setRootVisible(true);
                                                                                                                
        ActionMap map = getActionMap();
        map.put(DefaultEditorKit.copyAction, ExplorerUtils.actionCopy(explorerManager));
        map.put(DefaultEditorKit.cutAction, ExplorerUtils.actionCut(explorerManager));
        map.put(DefaultEditorKit.pasteAction, ExplorerUtils.actionPaste(explorerManager));
        map.put("delete", ExplorerUtils.actionDelete(explorerManager, true)); // NOI18N
                                                                                                                
        associateLookup( ExplorerUtils.createLookup(explorerManager, map) );
        explorerManager.setRootContext(exploredNode);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 400, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 300, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
    
    /**
     * Gets default instance. Do not use directly: reserved for *.settings files only,
     * i.e. deserialization routines; otherwise you could get a non-deserialized instance.
     * To obtain the singleton instance, use {@link findInstance}.
     */
    public static synchronized TodoListManagerTopComponent getDefault() {
        if (instance == null) {
            instance = new TodoListManagerTopComponent();
        }
        return instance;
    }
    
    /**
     * Obtain the TodoListManagerTopComponent instance. Never call {@link #getDefault} directly!
     */
    public static synchronized TodoListManagerTopComponent findInstance() {
        TopComponent win = WindowManager.getDefault().findTopComponent(PREFERRED_ID);
        if (win == null) {
            ErrorManager.getDefault().log(ErrorManager.WARNING, "Cannot find TodoListManager component. It will not be located properly in the window system.");
            return getDefault();
        }
        if (win instanceof TodoListManagerTopComponent) {
            return (TodoListManagerTopComponent)win;
        }
        ErrorManager.getDefault().log(ErrorManager.WARNING, "There seem to be multiple components with the '" + PREFERRED_ID + "' ID. That is a potential source of errors and unexpected behavior.");
        return getDefault();
    }
    
    public int getPersistenceType() {
        return TopComponent.PERSISTENCE_ALWAYS;
    }

    /** replaces this in object stream */
    public Object writeReplace() {
        return new ResolvableHelper();
    }
    
    protected String preferredID() {
        return PREFERRED_ID;
    }

    public ExplorerManager getExplorerManager() {
        return explorerManager;
    }
    
    final static class ResolvableHelper implements Serializable {
        private static final long serialVersionUID = 1L;
        public Object readResolve() {
            return TodoListManagerTopComponent.getDefault();
        }
    }
    
}
